# ======================================
# = Etiquetado en el paquete base de R =
# = L.F. González, T. Ortiz, sept 2010
# ======================================


# ============================
# = Mapa de correspondencias =
# ============================

library(ca)
data(author)
head(author)
analisis.corresp <- ca(author)
plot(analisis.corresp)

# =======================================
# = Un ejemplo más simple de etiquetado =
# =======================================
library(MASS) 
data(mammals)
head(mammals)

plot(log(mammals$body), log(mammals$brain), 
    col = "red", pch=20, cex=2)
     
abline(lm(log(brain)~log(body), data = mammals), 
    col="gray20", lwd=3)     

text(log(mammals$body), log(mammals$brain), 
    labels = rownames(mammals), 
    cex=1.1)



# ==========================================
# = En el paquete maptools está pointLabel =
# ==========================================
library(maptools)

plot(log(mammals$body), 
    log(mammals$brain),
    col = "red", pch=20, cex=2) 

abline(lm(log(brain)~log(body), 
    data=mammals), col="gray20", lwd=3)  

#sólo hay que cambiar text por pointLabel
pointLabel(log(mammals$body), 
    log(mammals$brain), 
    labels=rownames(mammals), 
    trace=TRUE, cex=1.0)

# =============================================
#  pointLabel funciona con recocido simulado,
#  explora el espacio de soluciones de etiquetado
# con un método monte carlo mediante cadenas de Markov.
# Intenta minimizar el traslape de las etiquetas
# =============================================     

pointLabel


# =====================================================
# = Con una modificación simple de pointLabel podemos
# = ver las iteraciones del método
# =====================================================
source("pointLabel_2.R")

pdf("optimizacion.pdf")
plot(log(mammals$body), log(mammals$brain), 
    col = "red", pch=20, cex=0.8) 
pointLabel_2(log(mammals$body), log(mammals$brain), 
    labels=rownames(mammals), 
    trace=TRUE, cex=1.0)   
dev.off()        


# =================================================
# = Ahora regresamos al mapa de correspondencias 
# = con el que comenzamos.
# =================================================

# Examinamos el comando plot
plot
# que es una función genérica
# ¿Qué funciones puede llamar?
methods("plot")


# Copiamos y modificamos para que use
# pointLabel en lugar de text
source("plot.ca.pointLabel.R")

#Y ahora graficamos de nuevo
analisis.corresp <- ca(author)
plot.ca.pointLabel(analisis.corresp)       
plot(analisis.corresp)

# ======================================
# = ¿Cómo usar pointLabel con ggplot2? =
# ======================================
library(ggplot2)

# Primero construimos la gráfica sin pointLabel
data(mammals)

#Necesitamos una columna con las etiquetas
mammals$etiq <- rownames(mammals)

#y calculamos antes el log de las variables
mammals$body.log <- log(mammals$body)
mammals$brain.log <- log(mammals$brain)
rownames(mammals) <- NULL
mammals

#Ahora construimos nuestra gráfica paso a paso
plot.1 <- ggplot(mammals, 
        aes(x = body.log, y = brain.log))

#Agregamos capa de puntos
plot.2 <- plot.1 + geom_point(colour = "red", 
    size = 3)
plot.2

#Ahora agregamos capa de etiquetas
plot.3 <- plot.2 + 
    geom_text(aes(x = body.log, y=brain.log, 
        label=etiq), 
        size=4.5, col="gray20")
plot.3

#Y finalmente la recta de regresión
plot.4 <- plot.3 + geom_smooth(method="lm")
plot.4                

# ===========================
# = Ahora usando pointLabel =
# ===========================

# Nos conviene definir primero la función siguiente,
# que agrega a la tabla de datos las coordenadas
# de las etiquetas según pointLabel

coord.etiquetas <- function(df, x, y, 
    etiq = "etiq", size = 3.5) {
        plot(df[, x], df[, y])
        salida.plabel <- pointLabel(df[, x], df[, y], 
            df[, etiq], 
            doPlot = TRUE, 
            cex = 2*size/5, xpd=TRUE)  
        dev.off()
        df$a <- salida.plabel$x
        df$b <- salida.plabel$y
        df
}    

#Agregamos posición de etiquetas
mammals.2 <- coord.etiquetas(mammals, 
        "body.log", "brain.log")

head(mammals.2)

#Ahora podemos repetir el ejemplo de arriba

#el primer cambio es pasamos mammals.2, etiquetada
plot.1 <- ggplot(mammals.2, 
        aes(x = body.log, y = brain.log))

plot.2 <- plot.1 + geom_point(colour = "red", 
    size = 3)
plot.2

#E indicamos las coordenadas de las etiquetas
plot.3 <- plot.2 + 
    geom_text(aes(x = a, y = b, label = etiq), 
        size=4.5, col="gray20")
plot.3

plot.4 <- plot.3 + geom_smooth(method = "lm")
plot.4

# ==========================================
# = En el siguiente ejemplo, veremos cómo 
# = hacer gráficas condicionales, usando también
# = pointLabel. 
# ==========================================              

#cargamos datos
datos <- read.csv(file="datos_ej.csv")
head(datos) 
datos

# primero componemos los nombres de
# las ciudades
simpleCap <- function(x) {
    s <- strsplit(x, " ")[[1]]
    paste(toupper(substring(s, 1,1)), 
          substring(s, 2),
          sep="", collapse=" ")  }

datos$plaza.2 <- sapply(
    datos$plaza, 
    function(st) simpleCap(tolower(st))
    )

head(datos)
datos

# Queremos graficar estimación Tipo 1 vs. Tipo 2,
# para todas las ciudades, y condicionando al segmento
# Necesitamos la librería reshape para poner
# los datos en la forma correcta

library(reshape)

# La función cast (junto con melt) son indispensables
# para convertir las tablas a la forma correcta
# que requieran nuestros gráficos
 
datos.cast <- cast(datos, 
    plaza.2 + segmento ~ tipo, value="media")

head(datos.cast)    

# Primer intento

q.1 <- ggplot(datos.cast, 
    aes(x=Tipo.1, y=Tipo.2)) + 
    geom_point()
q.1
#Condicionamos
q.2 <- q.1 + facet_wrap(~segmento)
q.2
#Etiquetas
q.3 <- q.2 + geom_text(aes(x=Tipo.1, y=Tipo.2, 
            label=plaza.2, colour=plaza.2), 
            size=4.5) 
q.3
# Ahora agregamos un un suavizador lowess
q.4 <- q.3 + geom_smooth(aes(group=1), span=3)
# la leyenda aquí no nos sirve:
q.5 <- q.4 + scale_colour_discrete(legend = F)         
q.5
                     
# =======================================
# = Ahora el problema son los traslapes
# = Podemos usar pointLabel, pero en cada
# = panel de la gráfica por separado
# =======================================

# Usamos el paquete plyr, que provee funciones
# para hacer dividir datos-aplicar a cada porción-
# volver a juntar.

#La función ddply toma un un Dataframe, devuelve
#un Dataframe y hace split-apply-combine

library(plyr)

mi.funcion <- function(df){
    coord.etiquetas(df, "Tipo.1", "Tipo.2",
        etiq = "plaza.2", size = 4.5) 
}

datos.cast.etiq <- ddply(datos.cast, 
    c("segmento"), mi.funcion)  

head(datos.cast.etiq)  

#Y nuestra gráfica ahora queda:
r.1 <- ggplot(datos.cast.etiq, 
    aes(x=Tipo.1, y=Tipo.2)) + 
    geom_point()
r.2 <- r.1 + facet_wrap(~segmento)
r.3 <- r.2 + geom_text(aes(x=a, y=b, 
            label=plaza.2, colour=plaza.2), 
            size=4.5) 
r.4 <- r.3 + geom_smooth(aes(group=1), span=3)
r.5 <- r.4 + scale_colour_discrete(legend = F)    
r.5
#comparamos 
q.5

#Obsérvese que podemos mejorar todavía
#si fijamos escalas comunes.
coord.etiquetas.2 <- 
    function(df, x, y, etiq = "etiq", size = 3.5) {
        plot(df[, x], df[, y], 
            xlim=c(0,0.9),ylim=c(0,0.9))
        orden <- pointLabel(df[, x], df[, y], 
                df[, etiq], 
                doPlot = TRUE, 
                cex = 2*size/5, xpd=TRUE)  
        dev.off()
        df$a <- orden$x
        df$b <- orden$y
        df
}       

mi.funcion.2 <- function(df){
    coord.etiquetas.2(df, "Tipo.1", "Tipo.2",
        etiq="plaza.2", size=4.5) 
}    


datos.cast.etiq.2 <- ddply(datos.cast, 
    c("segmento"), mi.funcion.2)  

head(datos.cast.etiq.2)  

#Y nuestra gráfica ahora queda:
s.1 <- ggplot(datos.cast.etiq.2, 
    aes(x=Tipo.1, y=Tipo.2)) + 
    geom_point()
s.2 <- s.1 + facet_wrap(~segmento)
s.3 <- s.2 + geom_text(aes(x=a, y=b, 
            label=plaza.2, colour=plaza.2), 
            size=4.5) 
s.4 <- s.3 + geom_smooth(aes(group=1), span=3)
s.5 <- s.4 + scale_colour_discrete(legend = F)    

#Comparamos
s.5
r.5
q.5
